###########################################################################
#
#   Copyright (c) 2001 Erik Enge <erik@thingamy.net>.
#   Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Id: klass.py,v 1.1.1.1 2001/07/02 09:29:10 erik Exp $
# $Source: /cvs/MakeProduct/klass.py,v $
__version__='$Revision: 1.1.1.1 $'[11:-2]

"""A bunch of classes for producing classes"""

import utils
import method
import property
import string

class basic_klass:
    """A basic klass"""

    def add_subklass(self, *args):
        """Add a subklass"""
        klass_ = apply(klass, args)
        klass_.setup_default()
        self.subklasses.append(klass_)

    def list_subklasses(self):
        """Return a list of all subklasses"""
        return self.subklasses

class method_klass(basic_klass):
    """A class that provides methods for manipulating methods"""

    def add_method(self, *args):
        """Add a method"""
        self.methods.append(apply(method.method, args))

    def list_methods(self):
        """Return a list methods"""
        return self.methods
    
    def remove_method(self, method_id):
        """Remove the method with the id 'method_id'"""
        instance = utils.find_entry(self.list_methods(), method_id)
        if instance: self.methods.remove(instance)
  
    def get_method(self, method_id):
        """Return the method with id 'method_id'"""
        return utils.find_entry(self.list_methods(), method_id)

class method_klass_rendrer(method_klass):
    """Adds methods for rendering"""

    def render_methods(self, indent=''):
        """Render all methods in this class"""
        methods = ""
        for method in self.list_methods():
            methods = methods + method.render_self(indent) + "\n"
        return methods 

    def render_methods_block(self, indent=''):
        """Render the block of text that contains the methods"""
        return self.render_methods(indent)

class property_klass(basic_klass):
    """A class that provides methods for manipulating properties"""

    def add_property(self, *args):
        """Add a property"""
        self.properties.append(apply(property.property, args))
        
    def list_properties(self):
        """Return a list of all properties"""
        return self.properties
    
    def remove_property(self, property_id):
        """Remove the property with id 'property_id'"""
        instance = utils.find_entry(self.list_properties(), property_id)
        if instance: self.properties.remove(instance)

    def get_property(self, property_id):
        """Get property with id 'property_id'"""
        return utils.find_entry(self.list_properties(), property_id)
                                    
class property_klass_rendrer(property_klass):
    """Adds methods for rendering"""

    def render_properties(self, indent=''):
        """Render a list of properties dictionaries"""
        properties = indent + ""
        for property in self.list_properties():
            if properties <> "":
                properties = properties + ",\n"
            properties = properties + indent + self.default_indent + \
                         property.render_self(self.default_indent)
        return properties

    def render_properties_dictionary(self, indent=''):
        """Render the _properties dictionary"""
        if string.strip(self.render_properties()):
            dictionary = indent + self.default_indent + "_properties = (\n"
            dictionary = dictionary + self.render_properties(indent) + "\n"
            dictionary = dictionary + indent + (2 * self.default_indent) + ")\n"
        else:
            dictionary = indent + self.default_indent + "_properties = ()\n"
        return dictionary

class klass_rendrer(method_klass_rendrer, property_klass_rendrer):
    """Adds methods for rendering"""

    def render_license(self):
        """Render the license"""
        return self.license

    def render_name_and_subclasses(self):
        """Render the class name and any subclasses"""
        text = "class " + self.id
        text = text + "(\n"
        text = text + self.default_indent + "Persistent,\n"
        text = text + self.default_indent + "Acquisition.Implicit,\n"
        text = text + self.default_indent + "product_base_item.product_base_item,\n"
        text = text + self.default_indent + "base_item.base_item):\n"
        return text

    def render_imports(self):
        """Return a string of needed imports"""
        text = "from Globals import HTMLFile, InitializeClass, Persistent\n"
        text = text + "from Products.WarpFramework import " +\
               "base_item, product_base_item\n"
        text = text + "import Acquisition\nimport AccessControl.Role\n"
        text = text + "import DateTime\n"
        for subklass in self.list_subklasses():
            text = text + "import " + subklass.id + "\n"
        return text

    def render_misc_definitions(self):
        """Render misc stuff"""
        text = self.default_indent + "meta_type = '" + self.meta_type + "'\n"
        return text + self.default_indent + "icon = '" + self.icon_path + "'\n"

    def render_meta_types_and_accessors(self):
        """Render the meta_types dictionary"""
        dict = "{'name': '%(id)s', 'action': '%(id)s.manage_add_%(id)s'}"
        if self.list_subklasses():
            text = self.default_indent + "meta_types = (\n"
            for subklass in self.list_subklasses():
                text = text + (2 * self.default_indent) + \
                       dict % {'id': subklass.id} + "\n"
            text = text + (2 * self.default_indent) + ")\n\n"
        else:
            text = ""
        return text

    def render_misc_headers(self):
        """Render misc headers"""
        text = ""
        if self.misc_headers:
            text = "# $Id: klass.py,v 1.1.1.1 2001/07/02 09:29:10 erik Exp $\n"
            text = text + "# $Source: /cvs/MakeProduct/klass.py,v $\n"
            text = text + "__version__ = '$Revision: 1.1.1.1 $'[11:-2]\n\n"
            text = text + self.module_docstring + "\n"
        return text

    def render_add_function(self):
        """Render the def *add() function and its HTMLFile"""
        text = "%(id)s_add_form = HTMLFile('dtml/%(id)s_add_form', globals())\n" % \
               {'id': self.id}
        text = text + "%(id)s_add = HTMLFile('dtml/%(id)s_add', globals())\n\n" % \
               {'id': self.id}
        text = text + """def manage_add_%(id)s(self, REQUEST=None, **kw):
    \"\"\"Add an instance of the %(id)s class\"\"\"
    id = str(DateTime.DateTime().time)[2:]
    self._setObject(id, %(id)s(id, kw))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)\n""" % {'id': self.id}
        return text

    def render_initialize_class(self):
        """Render the InitializeClass call"""
        return "InitializeClass(%s)" % self.id + "\n"

    def render_self(self):
        """Render the klass instance in text.  This will be
        used when the product class will write itself out to
        the filesystem."""
        text = self.render_license() + "\n"
        misc_headers = self.render_misc_headers()
        if misc_headers:
            text = text + misc_headers + "\n"
        imports = self.render_imports()
        if imports:
            text = text + self.render_imports() + "\n"
        text = text + self.render_add_function() + "\n"
        text = text + self.render_name_and_subclasses()
        text = text + self.default_indent + self.docstring + "\n\n"
        text = text + self.render_misc_definitions() + "\n"
        text = text + self.render_meta_types_and_accessors()
        text = text + self.render_properties_dictionary() + "\n"
        text = text + self.render_methods_block(self.default_indent)
        text = text + self.render_initialize_class()
        return text

class klass(method_klass, property_klass, klass_rendrer):
    """A klass"""

    def __init__(self, id, meta_type='', subklasses=[], license='',
                 icon_path='', misc_headers=1, docstring='"""doc string"""',
                 module_docstring='"""docstring"""', default_indent='    '):
        """Initialize an instance of the class"""
        self.id = id
        self.methods = []
        self.properties = []
        self.default_indent = default_indent
        self.license = license
        self.docstring = docstring
        self.subklasses = subklasses
        self.meta_type = meta_type
        self.icon_path = icon_path
        self.misc_headers = misc_headers
        self.module_docstring = module_docstring

    def add__setPropValue(self):
        """Add the _setPropValue method"""
        self.add_method('_setPropValue', ['self', 'property', 'value'],
                        "    eval(property + ' = ' + value)",
                        '''"""Add 'value' to 'property'"""''')

    def setup_default(self):
        """Add default methods and properties"""
        self.add__setPropValue()
        body = self.default_indent + "self.id = id\n\n"
        body = body + self.default_indent + "for property in self._properties:\n"
        body = body + (2 * self.default_indent) + \
               "if property['id'] in values.keys():\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], values[property['id']])\n"
        body = body + (2 * self.default_indent) + \
            "elif property.has_key('default'):\n"
        body = body + (3 * self.default_indent) + \
                "self._setPropValue(property['id'], property['default'])\n"

        self.add_method('__init__', ['self', 'id', ['values', {}]], body,
                        '"""Initialize an instance of the class"""')

    def write_subklasses(self, directory):
        """Write all subclasses"""
        for subklass in self.list_subklasses():
            subklass.write_self(directory)
                        
    def write_self(self, directory):
        """Write the rendered klass to a file"""
        file = open(directory + self.id + '.py', 'w')
        file.write(self.render_self())
        file.close()
        self.write_subklasses(directory)





