###########################################################################
#
#   Copyright (c) 2001 Erik Enge <erik@thingamy.net>.
#   Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Id: product.py,v 1.1.1.1 2001/07/02 09:29:10 erik Exp $
# $Source: /cvs/MakeProduct/product.py,v $
__version__='$Revision: 1.1.1.1 $'[11:-2]

"""A bunch of classes that mimick Zope Python Product behaviour.  Some
refactoring has been going on, and adding coding for ZClass Products
should't take too long -- for anyone interested ;)"""

import utils
import klass
import os

class basic_product:
    """A basic product"""
    pass

class klass_product(basic_product):
    """This class provides methods for manipulating classes"""

    def add_klass(self, *args):
        """Add a klass instance"""
        klass_ = apply(klass.klass, args)
        klass_.setup_default()
        self.klasses.append(klass_)

    def list_klasses(self):
        """Return a list of all klass instances"""
        return self.klasses

    def remove_klass(self, klass_id):
        """Remove the klass with the id 'klass_id'"""
        instance = utils.find_entry(self.list_klasses(), klass_id)
        if instance: self.klasses.remove(instance)          

    def get_klass(self, klass_id):
        """Return the klass instance with id 'klass_id'"""
        return utils.find_entry(self.list_klasses(), klass_id)

class zope_product(klass_product):
    """A class that provides methods for setting up a Zope Python
    Product"""

    def __init__(self, id, directory='', license='',
                 docstring='', default_indent='    '):
        """Initialize an instance of the class"""
        self.id = id
        self.license = license
        self.docstring = docstring
        self.default_indent = default_indent
        self.klasses = []
        # 'directory' is assumed to be an absolute path;
        # /usr/local/zope/lib/python/Products/
        self.directory = directory + self.id + '/'
            
    def add___init__py(self):
        """Add __init__.py file to the product directory"""
        if self.list_klasses():
            klass = self.list_klasses()[0].id
        module = klass
        file = open(self.directory + '__init__.py', 'w')
        file.write(open('templates/__init__.py').read() % \
                   {'license': self.license, 'id': self.id,
                    'klass': klass, 'module': module})
        file.close()

    def add_versiontxt(self):
        """Add version.txt file to the product directory"""
        file = open(self.directory + 'version.txt', 'w')
        file.write('0.0.0')
        file.close()

    def add_help_directory(self):
        """Add help/ directory"""
        os.mkdir(self.directory + 'help')

    def add_www_directory(self):
        """Add www/ directory"""
        os.mkdir(self.directory + 'www')

    def add_dtml_directory(self):
        """Add dtml/ directory"""
        os.mkdir(self.directory + 'dtml')

    def create_directory(self):
        """Create the directory in Products/"""
        # This method needs more code to test if the directory is
        # there alread, and if so, remove contents.  Maybe we should
        # ask the user before that, though.  Yeah, maybe we could
        # raise an error, and tools like SmartWizard could tell the
        # user.
        os.mkdir(self.directory)

    def setup_defaults(self):
        """Set up some defaults"""
        self.create_directory()
        self.add___init__py()
        self.add_versiontxt()
        self.add_help_directory()
        self.add_www_directory()
        self.add_dtml_directory()
            
    def write_klasses(self):
        """Write all klasses to files in self.directory"""
        for klass in self.list_klasses():
            klass.write_self(self.directory)

    def write_self(self):
        """Write the product to the filesystem"""
        self.setup_defaults()
        self.write_klasses()
